searchData={"items":[{"type":"module","title":"Reed","doc":"Streaming RSS parser with a built-in `Req` plugin for network-enabled chunked streaming.","ref":"Reed.html"},{"type":"module","title":"Installation - Reed","doc":"```elixir\ndef deps do\n  [\n    {:reed, \"~> 0.1.0\"}\n  ]\nend\n```\n\nReed implements a Sax-based parser for RSS feeds using the [`Saxy`](https://github.com/qcam/saxy) library.\n\nYou can manually use the `Reed.Handler` (which implements the `Saxy.Handler` behaviour) with `Saxy` to parse\nstrings or from `Stream`s, but the killer feature of `Reed` is the `Reed.ReqPlugin` module, which powers the top-level\n`Reed.get` / `Reed.get!` API.\n\n`Reed` began as a need for a way to read RSS feeds by first reading the feed-level metadata followed\nby item-by-item streaming without loading the entire feed into memory, all while doing so from a remote URL.\n\n`Reed` combines the `Saxy.Partial` module with `Req`'s streaming `:into` option to do just that.\n\n`Reed.ReqPlugin` takes advantage of `Req`'s chunking capability to parse RSS feeds directly from over the network, applying\ntransformation functions to each RSS item lazily.\n\nThis means you do not have to store the entire RSS feed in memory or on disk to convert to a traditional Elixir `Stream`\n(as is required to use `Saxy.parse_stream/4`), but instead directly uses `Saxy.Partial` to parse chunk-by-chunk directly\nover the wire.\n\nThe `Reed.Transformers` module provides some convenient transformation functions to be used during the parsing.\n\nThe transformation pipeline is invoked whenever a new RSS item is read, and works with an accumulating state that persists\nduring the entire RSS read.\n\n`Reed` provides a dead-simple API that also allows for flexible handling of items during the stream through\nthe use of transformation pipelines (see `Reed.Transformers`).  These pipelines define how to handle the item\nstream, and function as a reduction over an input state containing feed-level metadata, the current item, and\na private field where you can store other data.\n\nThe state also maintains a `:halted` field that controls whether to halt the stream after the current item has been\nprocessed (had the whole `:transform` pipeline applied).\n\nYou can also control when to move on to the next item during a transformation pipeline by returning either `:halt`\nor `{:halt, state}` from any step in the transformation pipeline (see `Reed.Transformers.filter/2` for an example).\n\nYou can compose the built-in pipeline function from `Reed.Transformers` or create your own unique steps to create very\nsimple yet powerful parsing instructions to carefully read only the exact parts of the RSS stream that you're interested in.","ref":"Reed.html#module-installation"},{"type":"module","title":"Examples - Reed","doc":"#","ref":"Reed.html#module-examples"},{"type":"module","title":"Get the feed metadata - Reed","doc":"```elixir\nimport Reed.Transformers\nReed.get!(rss_url, transform: transform(halt()))\n```\n\n#","ref":"Reed.html#module-get-the-feed-metadata"},{"type":"module","title":"Get all items in a list - Reed","doc":"```elixir\nimport Reed.Transformers\nReed.get!(rss_url, transform: transform(collect()))\n```\n\n#","ref":"Reed.html#module-get-all-items-in-a-list"},{"type":"module","title":"Get the first 5 items in a list - Reed","doc":"```elixir\nimport Reed.Transformers\nReed.get!(rss_url, transform: collect() |> limit(5) |> transform())\n```\n\n#","ref":"Reed.html#module-get-the-first-5-items-in-a-list"},{"type":"module","title":"Get all `itunes:` namespaced elements from the first 2 items as a list - Reed","doc":"```elixir\nimport Reed.Transformers\n\nReed.get!(rss_url,\n  transform:\n    transform_item(\n      &Map.filter(&1, fn\n        {<<\"itunes:\", _rest::binary>>, _v} -> true\n        _ -> false\n      end)\n    )\n    |> collect()\n    |> limit(2)\n    |> transform()\n)\n```","ref":"Reed.html#module-get-all-itunes-namespaced-elements-from-the-first-2-items-as-a-list"},{"type":"function","title":"Reed.get/2","doc":"Stream the RSS feed at the specified URL.","ref":"Reed.html#get/2"},{"type":"function","title":"Options - Reed.get/2","doc":"* `:transform` - The transformation pipeline to apply while streaming the RSS feed's items.  Accepts either a single `transformer`\nor a list of `transformer`s\n Accepts passes through option to `Req`.","ref":"Reed.html#get/2-options"},{"type":"function","title":"Reed.get!/2","doc":"","ref":"Reed.html#get!/2"},{"type":"module","title":"Reed.ReqPlugin","doc":"A `Req` plugin for `Reed`.\n\nThis will stream the RSS feed over the network and apply the transformation to each item lazily.\n\nThis will store the result into the `Req.Response` struct under the `:rss` key in the `:private`\nfield.\n\nThis is streamed chunk-by-chunk, meaning you can stop reading the RSS feed at any point, and\nyou only store in memory what you decide to using the `:transform` option.\n\nYou can get the result using `Req.Response.get_private(response, :rss)`.","ref":"Reed.ReqPlugin.html"},{"type":"module","title":"Options - Reed.ReqPlugin","doc":"* `:transform` - The transformation function / pipeline to apply to each item in the RSS feed. Check the documentation for `Reed` for more information.","ref":"Reed.ReqPlugin.html#module-options"},{"type":"function","title":"Reed.ReqPlugin.attach/2","doc":"Attaches `Reed.ReqPlugin` to the given `Req.Request` struct.","ref":"Reed.ReqPlugin.html#attach/2"},{"type":"function","title":"Reed.ReqPlugin.setup_rss_stream/1","doc":"","ref":"Reed.ReqPlugin.html#setup_rss_stream/1"},{"type":"module","title":"Reed.State","doc":"The state tracked when reading an RSS stream with `Reed`.","ref":"Reed.State.html"},{"type":"module","title":"Fields - Reed.State","doc":"#","ref":"Reed.State.html#module-fields"},{"type":"module","title":"Public Fields - Reed.State","doc":"These fields are considered public and are passed to the pipeline transformation steps:\n* `:feed_info` - The feed-level metadata from the RSS feed. This will be collected before any items are parsed, and will\n  be available during each transformation step.\n* `:current_item` - Always references stores the current item in the stream.\n* `:halted` - Whether to halt the stream after the current item is processed. Defaults to `false`.\n* `:private` - A private map field that can be used during the transformation pipeline. Defaults to an empty map.\n\n#","ref":"Reed.State.html#module-public-fields"},{"type":"module","title":"Private Fields - Reed.State","doc":"These fields are considered private and are used internally by `Reed` during the parsing process. These are NOT passed to\nthe pipeline transformation functions.\n\n* `:current_text`\n* `:current_path`\n* `:transform`","ref":"Reed.State.html#module-private-fields"},{"type":"type","title":"Reed.State.t/0","doc":"","ref":"Reed.State.html#t:t/0"},{"type":"module","title":"Reed.Transformers","doc":"Convenient transformer functions to be used during parsing with `Reed.Handler`.\n\nWhen composing functions from `Reed.Transformers`, be aware that order matters\nand is preserved.\n\nFor example, if you use `transform/2` and `collect/2` in your pipeline,\nin order for `collect/2` to collect the transformed version of the item it\nmust come after `transform/2`.\n\nEach transformer function must accept a `state` as the first argument and must\nreturn either a new `state` or a return value matching those accepted by\n`Enum.reduce_while/3`.","ref":"Reed.Transformers.html"},{"type":"function","title":"Reed.Transformers.collect/1","doc":"Collects all items into a list.\n\nThe final list will be under the `:items` key in the state, and\nwill be in reverse order.","ref":"Reed.Transformers.html#collect/1"},{"type":"function","title":"Reed.Transformers.filter/2","doc":"Filters out items according to the `filter_with` function. This will skip all remaining\nsteps in the pipeline for the current item. This does NOT halt the reading overall, but\nrather only the current transformation pipeline for the current item.\n\nThis means that its position in the pipeline has effects on downstream operations,\n(eg. if you call `limit` before `filter` it will count all items, but\nif you call `limit` after `filter` it will only count the filtered items).","ref":"Reed.Transformers.html#filter/2"},{"type":"function","title":"Reed.Transformers.halt/1","doc":"Halt the RSS streaming after the current item.","ref":"Reed.Transformers.html#halt/1"},{"type":"function","title":"Reed.Transformers.limit/2","doc":"Keeps track of how many RSS feed items have been processed and will halt\nthe stream after the given # of RSS feed items have been processed.","ref":"Reed.Transformers.html#limit/2"},{"type":"macro","title":"Reed.Transformers.pipeline/1","doc":"A convenient macro that will format take your pipeline correctly\nif you have a pipeline composed entirely of functions with the following\nfunction signature: `(state::state, opts::list() \\ [])`\n\nThis is the function signature that all transformer functions within\n`Reed.Transformers` have.\n\nThis macro should always be called last in a transformation pipeline.","ref":"Reed.Transformers.html#pipeline/1"},{"type":"macro","title":"Example - Reed.Transformers.pipeline/1","doc":"The following two are equivalent:\n\n```elixir\nReed.get!(url, transform: [\n    fn state -> stop_after(state, 1),\n    fn state -> collect(state)\n  ]\n)\n```\n\n```elixir\nReed.get!(url, transform:\n  fn state ->\n  state\n  |> stop_after(1)\n  |> collect()\nend)\n```\n\n```elixir\nReed.get!(url, transform:\n  stop_after(1)\n  |> collect()\n  |> pipeline()\n)\n```","ref":"Reed.Transformers.html#pipeline/1-example"},{"type":"function","title":"Reed.Transformers.transform/2","doc":"Transforms the current RSS feed item according to the\ngiven an arity-1 transformation function.","ref":"Reed.Transformers.html#transform/2"},{"type":"type","title":"Reed.Transformers.transformer/0","doc":"","ref":"Reed.Transformers.html#t:transformer/0"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,49,46,50]}}